import React, { useCallback, useReducer, useRef } from 'react';
import { useIsomorphicLayoutEffect } from './utils';
import Context from './Context';
import applyMiddleware from './applyMiddleware';
import bindActionCreators from './bindActionCreators';

const Provider = (props) => {
  const initialState = props.rootReducer(props.initialValue || {}, { type: '__INIT__' });

  const [state, dispatch] = useReducer(props.rootReducer, initialState);

  const subscribers = useRef([]);

  const stateRef = useRef(initialState);

  stateRef.current = state;

  const getState = () => {
    return stateRef.current;
  };

  const interceptDispatch = applyMiddleware({ dispatch, getState }, props.rootMiddleware);

  const boundActions = bindActionCreators({ dispatch: interceptDispatch, getState }, props.rootAction || {});

  useIsomorphicLayoutEffect(() => {
    subscribers.current.forEach(fn => fn(state));
  }, [state]);

  const subscribe = useCallback((fn) => {
    if (typeof fn === 'function' && subscribers.current.indexOf(fn) === -1) {
      subscribers.current.push(fn);
    }

    return () => {
      const index = subscribers.current.indexOf(fn);

      if (index !== -1) {
        subscribers.current.splice(index, 1);
      }
    };
  }, []);


  const value = {
    state: getState(),
    dispatch: interceptDispatch,
    actions: boundActions,
    subscribe
  };

  return (
    <Context.Provider value={value} {...props} />
  );
};

export default Provider
